<?php

namespace Nest\Model;

abstract class EntityPropertyBlueprint
{
	protected string $class;

	protected bool $multiple;

	protected bool $allowInline = true;

	protected EntityPropertyMode $mode;

	protected ?string $localKey = null;

	protected ?string $relatedKey = null;

	protected ?string $pivotTable = null;

	protected ?string $pivotLocalKey = null;

	protected ?string $pivotRelatedKey = null;

	protected bool $eagerLoad = false;

	public function class(string $class): static
	{
		$this->class = $class;
		return $this;
	}

	public function multiple(bool $multiple = true): static
	{
		$this->multiple = $multiple;
		return $this;
	}

	/**
	 * Set if inline loading of the relation is allowed or not.
	 * @param bool $inline True to allow relation inline loading.
	 * @return $this
	 */
	public function allowInline(bool $inline = true): static
	{
		$this->allowInline = $inline;
		return $this;
	}

	public function fromLocal(?string $localKey = null, ?string $relatedKey = null): static
	{
		$this->mode = EntityPropertyMode::LOCAL;
		$this->localKey = $localKey;
		$this->relatedKey = $relatedKey;
		return $this;
	}
	public function fromRelated(?string $relatedKey = null, ?string $localKey = null): static
	{
		$this->mode = EntityPropertyMode::RELATED;
		$this->localKey = $localKey;
		$this->relatedKey = $relatedKey;
		return $this;
	}

	public function localKey(string $localKey): static
	{
		$this->localKey = $localKey;
		return $this;
	}
	public function relatedKey(string $relatedKey): static
	{
		$this->relatedKey = $relatedKey;
		return $this;
	}

	public function fromPivot(string $pivotTable, ?string $pivotLocalKey = null, ?string $pivotRelatedKey = null): static
	{
		$this->mode = EntityPropertyMode::PIVOT;
		$this->pivotLocalKey = $pivotLocalKey;
		$this->pivotRelatedKey = $pivotRelatedKey;
		return $this;
	}

	public function pivotLocalKey(string $pivotLocalKey): static
	{
		$this->pivotLocalKey = $pivotLocalKey;
		return $this;
	}
	public function pivotRelatedKey(string $pivotRelatedKey): static
	{
		$this->pivotRelatedKey = $pivotRelatedKey;
		return $this;
	}

	/**
	 * Set if the related field should be eager loaded or not.
	 * Only apply on related fields.
	 * @param bool $load True to eager load the field.
	 * @return $this
	 */
	public function eagerLoad(bool $load = true): static
	{
		$this->eagerLoad = $load;
		return $this;
	}
}
